#!/bin/bash
#
# Description: 配列展開の基本パターン5
#              デフォルト値1
# Todo: インデックスを指定したパターンも作成
#       分量が多いので分割
# TBD: 日本語文字列いれるか

source pre.bash

echo "デフォルト値1"

## Default
:<< EOC
# Use a default value: Arrays 配列のデフォルト値
配列の場合、動作は非常に似ています。 繰り返しになりますが、個々の要素を特定のインデックスで展開することと、@と*の添字を使用して配列をまとめて展開することの間には違いがあります。

個々の要素についても、まったく同じです。展開された要素がNULLまたは設定解除されている場合（： - と - の変形を参照）、デフォルトのテキストは展開されます。
大量展開構文の場合、配列が次の場合はデフォルトのテキストが展開されます。
要素が含まれていないか、設定されていません（： - と - の変形はここでも同じ意味です）。
nullstringである要素のみを含みます（： -  variant）
言い換えれば、この展開型の基本的な意味は、可能な限り配列に適用されます。
EOC

echo "    デフォルト値"
unset array
echo "  arrayは、未定義。"
echo "  \${array[@]:-未定義かも} => ${array[@]:-未定義かも}"
echo "  \${array[@]-未定義かも} => ${array[@]-未定義かも}"
echo


array=()
echo "  array=()"
echo "  \${array[@]:-未定義かも} => ${array[@]:-未定義かも}"
echo "  \${array[@]-未定義かも} => ${array[@]-未定義かも}"
echo

array=("")
echo '  array=("")'
echo "  \${array[@]:-未定義かも} => ${array[@]:-未定義かも}"
echo "  \${array[@]-未定義かも} => ${array[@]-未定義かも}"
echo

array=("" word)
echo '  array=("" word)'
echo "  \${array[@]:-未定義かも} => ${array[@]:-未定義かも}"
echo "  \${array[@]-未定義かも} => ${array[@]-未定義かも}"
echo

:<< EOC
# Assign a default value: Arrays
配列の場合、この拡張タイプは制限されています。 個々のインデックスでは、「通常の」パラメータの場合と同じように動作し、デフォルト値がこの1つの要素に割り当てられます。 質量拡張添字の@と*は、代入できないため、ここでは使用できません。
EOC
echo "    デフォルト値の代入"
unset array
echo "    arrayは、未定義。"
echo "    \${array[@]:=未定義かも} => ${array[@]:=未定義かも}"
echo "    \${array[@]=未定義かも} => ${array[@]=未定義かも}"
echo

array=()
echo "    array=()"
echo "    \${array[@]:=未定義かも} => ${array[@]:=未定義かも}"
echo "    \${array[@]=未定義かも} => ${array[@]=未定義かも}"
echo

array=("")
echo '    array=("")'
echo "    \${array[@]:=未定義かも} => ${array[@]:=未定義かも}"
echo "    \${array[@]=未定義かも} => ${array[@]=未定義かも}"
echo

array=("" word)
echo '    array=("" word)'
echo "    \${array[@]:=未定義かも} => ${array[@]:=未定義かも}"
echo "    \${array[@]=未定義かも} => ${array[@]=未定義かも}"
echo

## Alternate
:<< EOC
# Use an alternate value: Arrays
配列がデフォルト値に展開される場合と同様に、この展開は、インデックスで個々の配列要素を使用するときの「通常の」パラメータのように動作しますが、質量展開添字@と*を使用するときの反応は異なります。

個々の要素についても、まったく同じです。展開された要素がNULLでも設定解除されてもいない場合（：+と+の派生形を参照）、代替テキストは展開されます。
大量展開構文の場合、配列が次の場合は代替テキストが展開されます。
minの要素が含まれます。 1つの要素がnullstringではありません（：+と+の異形はここでも同じ意味です）
nullstringではない要素のみを含みます（：+変種）
場合によっては、デフォルト値を使用するための説明のコード例を参照してください。
EOC

echo "    定義されていたら、別の値を使う"
unset array
echo "    array is not defined"
echo "    \${array[@]:+未定義かも} => ${array[@]:+未定義かも}"
echo "    \${array[@]+未定義かも} => ${array[@]+未定義かも}"
echo

array=()
echo "    array=()"
echo "    \${array[@]:+未定義かも} => ${array[@]:+未定義かも}"
echo "    \${array[@]+未定義かも} => ${array[@]+未定義かも}"
echo

array=("")
echo '    array=("")'
echo "    \${array[@]:+未定義かも} => ${array[@]:+未定義かも}"
echo "    \${array[@]+未定義かも} => ${array[@]+未定義かも}"
echo

array=("" word)
echo '    array=("" word)'
echo "    \${array[@]:+未定義かも} => ${array[@]:+未定義かも}"
echo "    \${array[@]+未定義かも} => ${array[@]+未定義かも}"
echo

