#!/bin/bash
#
# Subject: 2次元配列の正しい記述の仕方を探る


echo '2次元配列を擬似的に実現する'
echo

arr0=(100 200 300) arr1=(10 20 30)
l=1 c=2

echo "arr0[@]="${arr0[@]}
echo "arr1[@]="${arr1[@]}
echo
echo "l=$l, c=$c"
echo

echo "インデックス値を直接指定して2次元配列のようにアクセス"
echo '  ${arr1[2]} => ' "${arr1[2]}"
echo

echo "補題: 以下の5通りのいずれの記法だとアクセスできるか確認する。"
echo -n '   1.  ${arr$l[$c]} => '     ; echo "${arr$l[$c]}"
echo -n '   2.  ${arr1[$c]} => '      ; echo "${arr1[$c]}"
echo -n '   3.  ${arr$l[2]} => '      ; echo "${arr$l[2]}"
echo -n '   4.  ${arr${l}[${c}]} => ' ; echo "${arr${l}[${c}]}"
echo -n '   5.  \${arr$l[$c]} => '    ; echo "\${arr$l[$c]}"
echo "  結果、2.の記法(1次元配列的アクセス)は可能。"
echo "  5.では要素にアクセスできない。他の記法は認められない。"
echo

echo "正しくは、2つのインデックスを変数として参照するにはevalを用いる。"
echo '  参照の方法:'
echo $'  $ eval "\${arr$l[$c]}" '
echo -n '    実行結果: '
eval echo "\${arr$l[$c]}"
echo

echo '  参考: インデックスのみ展開、表示する:'
echo $'  $ eval echo \\\'"\${arr$l[$c]}"\\\' '
echo -n '    実行結果: '
eval echo \'"\${arr$l[$c]}"\'
echo

echo '  2次元配列の要素を、変数(port)に代入する場合:'
eval port=\${arr$l[$c]}
echo '  $ eval port=\${arr$l[$c]}'
echo -n '   実行結果: '
echo 'port =>' $port
echo

echo '別解: 連想配列を用いれば簡単に2次元配列を作れる'
arr0=(100 200 300) arr1=(10 20 30)
declare -A aa=([0,0]=100 [0,1]=200 [0,2]=300 [1,0]=10 [1,1]=20 [1,2]=30)
echo "${aa[@]@A}"
echo '${aa[@]} => ' "${aa[@]}"
echo '  ${aa[$l,$c]} => ' "${aa[$l,$c]}"

